import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

/*
 * Created on Dec 16, 2004
 *
 * Copyright (c) Sodius SAS
 * 
 */

/**
 * This class adds some convenience methods for the Rhapsody.ini.
 * 
 * @author Thomas Capelle
 * 
 */
public class RhapsodyIniFile extends IniFile {
	
	public final static String RHAPSODY_INI_WIN = "rhapsody.ini";
	public final static String RHAPSODY_INI_LINUX = "rhapsody.exe.ini";
	protected final static String NAME_HELPERS = "Helpers";
	protected final static String NAME_NUMBER_OF_ELEMENTS = "numberOfElements";
	protected final static String NAME_NAME = "name";
	protected final static String NAME_COMMAND = "command";
	protected final static String NAME_ARGUMENTS = "arguments";
	protected final static String NAME_INITIAL_DIR = "initialDir";
	protected final static String NAME_DLL_SERVER_COMPATIBLE = "DLLServerCompatible";
	protected final static String NAME_IS_VISIBLE = "isVisible";
	protected final static String NAME_IS_MACRO = "isMacro";
	protected final static String NAME_APPLICABLE_TO = "applicableTo";
	protected final static String NAME_PLUGIN = "Plugin";
	protected final static String NAME_PLUGINS = "Plugins";
	protected final static String NAME_VERSION = "RhpVersion";
	protected final static String NAME_DLL = "DLL";
	protected final static String NAME_JAVA_MAIN_CLASS = "JavaMainClass";
	protected final static String NAME_LOAD_EVENT = "loadEvent";
    protected final static String NAME_MENU = "MenuItem";
    protected final static String NAME_CLASSPATH = "JavaClassPath";
    protected final static String NAME_LIBPATH = "JavaLibPath";
    public final static String EXTERNALCG = "ExternalCG";
    public final static String MENU = "Menu";
    
	private int numberOfElements = 0;

	private boolean modifiedHelpers = false;
	
	private final String rhapsody_ini_name;
	
	/**
	 * First searches in the rhapsody directory and then in the windows system directory
	 * @param winDir : The windows directory
	 * @param rhapDir : The rhapsody install directory - can be null
	 * @throws IOException
	 */
	public RhapsodyIniFile(String winDir, String rhapDir, boolean save) throws IOException {
		super();
		rhapsody_ini_name = System.getProperty("os.name").startsWith("Windows") ? RHAPSODY_INI_WIN : RHAPSODY_INI_LINUX;
		includeEmptyAssignments = true;
		saveOnChange = save;
		String name = "";
		if (rhapDir != null) {
			name = rhapDir + File.separator + rhapsody_ini_name;
		}
		if (rhapDir == null || !((new File(name)).exists())) {
			name = winDir + File.separator + rhapsody_ini_name;
			if (!((new File(name)).exists()))
				throw new FileNotFoundException();
		}
		fileName = name;
		loadFile();
		parseLines();
	}

	/**
	 * @param name
	 * @param save
	 * @param create
	 * @throws IOException
	 */
	public RhapsodyIniFile(String winDir, boolean save) throws IOException {
		this(winDir, null, save);
	}
	

	/*
	 * @see IniFile#parseLines()
	 */
	protected void parseLines() {
		super.parseLines();
		// loads "Helpers" section
		String val = this.getValue(NAME_HELPERS, NAME_NUMBER_OF_ELEMENTS);
		if (val.equals(""))
			val = "0";
		numberOfElements = Integer.parseInt(val);
		if (numberOfElements < 0)
			numberOfElements = 0;
	}

	protected void addHelper(String name, String command, String arguments, String initialDir, String isDLLServerCompatible, String isVisible,
			String isMacro, String applicableTo, int number) throws IOException {

		this.setValue(NAME_HELPERS, NAME_NAME + number, name);
		this.setValue(NAME_HELPERS, NAME_COMMAND + number, command);
		this.setValue(NAME_HELPERS, NAME_ARGUMENTS + number, arguments);
		this.setValue(NAME_HELPERS, NAME_INITIAL_DIR + number, initialDir);
		this.setValue(NAME_HELPERS, NAME_DLL_SERVER_COMPATIBLE + number, isDLLServerCompatible);
		this.setValue(NAME_HELPERS, NAME_IS_VISIBLE + number, isVisible);
		this.setValue(NAME_HELPERS, NAME_IS_MACRO + number, isMacro);
		this.setValue(NAME_HELPERS, NAME_APPLICABLE_TO + number, applicableTo);

		modifiedHelpers = true;
	}
		
	/**
	 * Adds a helper to the list of helpers.  Looks first for an existing helper of the
	 * same name, and if found, uses that number.  If not found, the numberOfElements is
	 * incremented, and the new number is used.
	 * @param name
	 * @param command
	 * @param arguments
	 * @param initialDir
	 * @param isVisible
	 * @param isMacro
	 * @throws IOException
	 */
	public void addHelper(String name, String command, String arguments, String initialDir, String isDLLServerCompatible, String isVisible,
			String isMacro, String applicableTo) throws IOException {
		
		int number = this.getKeyNumber(NAME_HELPERS, name);
		if (number < 0) {
			number = ++numberOfElements;
			this.setValue(NAME_HELPERS, NAME_NUMBER_OF_ELEMENTS, "" + numberOfElements);
		}
		this.addHelper(name, command, arguments, initialDir, isDLLServerCompatible, isVisible, isMacro, applicableTo, number);
		
	}
		
	/** Returns the key number that should be used for the elementName by
	 *  searching for the same name in the existing keys.  Searches for a maximum
	 *  of number of elements. */

	public int getKeyNumber(
		String sectionName,
		String elementName) {
		int i = 1;
		boolean foundTopic = false;

		while ((i <= numberOfElements) && (!foundTopic)) {
			String key = "name" + i;
			String value = this.getValue(sectionName, key);
			if (value.equals(elementName)) {
				foundTopic = true;
			} else {
				i = i + 1;
			}
		}

		if (foundTopic) {
			return i;
		} else {
			return -1;
		}
	}

    
    /* Adds a plugin with the following format.  First removes it if it already exists.
     * The name given to the plugin is [name][version].
    [Plugin]
    Plugins=RiACodeGen5.0.1
    [RiACodeGen5.0.1]
    name=RulesComposer_Generator
    RhpVersion=7.0
    MenuItem=
    JavaMainClass=com/sodius/RulesComposer/MyMainClass
    classpath=<Your path>
    libpath=<Your path>
     */
    
    public void addJavaPlugin(String name, String version, String menu, String mainClass,
    		String classPath, String libPath) throws IOException {
    	this.addJavaPlugin(name, version, menu, mainClass, classPath, libPath, "");
    }
    
    public void addJavaPlugin(String name, String version, String menu, String mainClass,
    		String classPath, String libPath, String loadEvent) throws IOException {
        
    // remove before to add
        this.removePlugin(name, version);
        
        String pluginList = this.getValue(NAME_PLUGIN, NAME_PLUGINS);
        pluginList = pluginList.replace(';',',');
        if (!pluginList.equals("") && !pluginList.endsWith(",")) {
            pluginList = pluginList + ",";
        }
        String pluginName = name + version;
        this.setValue(NAME_PLUGIN, NAME_PLUGINS, pluginList + pluginName);
        this.setValue(pluginName, NAME_NAME, name);
        this.setValue(pluginName, NAME_VERSION, version);
        this.setValue(pluginName, NAME_JAVA_MAIN_CLASS, mainClass);
        this.setValue(pluginName, NAME_MENU, menu);
        this.setValue(pluginName, NAME_CLASSPATH, classPath);
        this.setValue(pluginName, NAME_LIBPATH, libPath);
        if (loadEvent != null && !loadEvent.equals("")) {
        	this.setValue(pluginName, NAME_LOAD_EVENT, loadEvent);
        this.setValue(pluginName, NAME_DLL_SERVER_COMPATIBLE, "TRUE");
        }
    }

	public void removePlugin(String name, String version) throws IOException {
		String pluginName = name + version;
		this.deleteSubject(pluginName);
		String plugins = this.getValue(NAME_PLUGIN, NAME_PLUGINS);
		plugins = plugins.replace(';', ',');
		int i = plugins.indexOf(pluginName);
		if (i != -1) {
			if (i == 0) {
				if (plugins.length() > pluginName.length()) {
					// remove the following ','
					if (plugins.charAt(pluginName.length()) == ',') {
						plugins = plugins.substring(pluginName.length() + 1);
					} else {
						plugins = plugins.substring(pluginName.length());
					}
				} else {
					plugins = "";
				}
			} else {
				int x = 0; 
				if (plugins.charAt(i - 1) == ',') x = 1;
				plugins = plugins.substring(0, i - x)
				 + plugins.substring(i + pluginName.length());
			}
			plugins = plugins.trim();
			if (plugins.matches("[^a-zA-Z]*")) {
				this.deleteSubject(NAME_PLUGIN);
			}
			this.setValue(NAME_PLUGIN, NAME_PLUGINS, plugins);
		}
	}


	/**
	 * Removes a helper from the list of helpers.
	 * @param name
	 * @throws IOException
	 */
	public void removeHelper(String name) throws IOException {
		
		int number = this.getKeyNumber(NAME_HELPERS, name);
		if (number < 0) return; // not found
		
		// remove the elements
		this.removeHelper(number);

	}
		
	private void removeHelper(int number) throws IOException {
		
		if (number < 0) return; // not found
		
		// the only time it is save to decrement the numbers is
		// if I'm removing the last helper
		if (number == numberOfElements) numberOfElements--;
		
		// remove the elements
		this.deleteValue(NAME_HELPERS, NAME_NAME + number);
		this.deleteValue(NAME_HELPERS, NAME_COMMAND + number);
		this.deleteValue(NAME_HELPERS, NAME_ARGUMENTS + number);
		this.deleteValue(NAME_HELPERS, NAME_INITIAL_DIR + number);
		this.deleteValue(NAME_HELPERS, NAME_DLL_SERVER_COMPATIBLE + number);
		this.deleteValue(NAME_HELPERS, NAME_IS_VISIBLE + number);
		this.deleteValue(NAME_HELPERS, NAME_IS_MACRO + number);
		this.deleteValue(NAME_HELPERS, NAME_APPLICABLE_TO + number);
		
		modifiedHelpers = true;

	}
		
	/**
	 * Anytime I touch the helpers (add/remove), I need to redo
	 * the numbering in case i removed or added from the middle
	 * of the numbering.
	 */
	private void reorderHelpers() throws IOException {
		
		String [] vars = this.getVariables(NAME_HELPERS);
		int max = 0;
		
		// collect all the numbers
		for (int i = 0; i < vars.length; i++) {
			// only look for names
			if (vars[i].startsWith(NAME_NAME)) {
				String numbString = vars[i].replaceFirst("[^0-9]*", "");
				try {
					int numb = Integer.parseInt(numbString);
					if ( numb > max) {
						max = numb;
					}
				} catch (NumberFormatException e) {
					// do nothing
				}
			}
		}
		
		int skipped = 0;
		for (int i = 1; i <= max; i++) {
			String value = this.getValue(NAME_HELPERS, NAME_NAME + i);
			if (value.length() == 0) {
				// this number is not used
				skipped++;
			} else {
				// change the number if necessary
				if (skipped > 0) {
					this.addHelper(value,
							this.getValue(NAME_HELPERS, NAME_COMMAND + i),
							this.getValue(NAME_HELPERS, NAME_ARGUMENTS + i),
							this.getValue(NAME_HELPERS, NAME_INITIAL_DIR + i),
							this.getValue(NAME_HELPERS, NAME_DLL_SERVER_COMPATIBLE + i),
							this.getValue(NAME_HELPERS, NAME_IS_VISIBLE + i),
							this.getValue(NAME_HELPERS, NAME_IS_MACRO + i),
							this.getValue(NAME_HELPERS, NAME_APPLICABLE_TO + i),
							i - skipped);
				}
			}
			
		}
		
		for (int i = max; i > max - skipped; i--) {
			this.removeHelper(i);
		}
		
		numberOfElements = max - skipped;

	}
		
	/*
	 * @see IniFile#saveFile()
	 */
	public void saveFile() throws IOException {
		if (modifiedHelpers) {
			this.reorderHelpers();
			modifiedHelpers = false;
		}
		this.setValue(NAME_HELPERS, NAME_NUMBER_OF_ELEMENTS, "" + numberOfElements);
		super.saveFile();
	}
}
