package com.sodius.mdw.log.reporter;

import java.io.IOException;
import java.io.Writer;
import java.util.LinkedList;
import java.util.List;

import com.sodius.mdw.log.EventSeverity;
import com.sodius.mdw.log.EventStatus;
import com.sodius.mdw.log.reporter.tree.SubTreeGenerator;
import com.sodius.mdw.log.reporter.tree.Tree;


/**
 * @author Mickael Albert
 * Reporter using a tree of event for computing number of children and generating a summary
 *
 */
public class SummarizedLogReporter extends TextLogReporter {

	private final SubTreeGenerator treeGen;
	private final boolean isFlat;


	/**
	 * @param minimumSeverity
	 * @param maximumSeverity
	 * @param writer
	 * @param header
	 * @param footer
	 * @param treeGen the generator of the tree used for computing summary
	 * @param isFlat if true, generates a flat report, only branches without children will be displayed. If false, generate a textual tree
	 * @throws IOException
	 */
	public SummarizedLogReporter(EventSeverity minimumSeverity,
			EventSeverity maximumSeverity, Writer writer, String header, String footer, SubTreeGenerator treeGen, boolean isFlat) throws IOException
	{
		super(minimumSeverity, maximumSeverity, writer, header, footer);
		this.treeGen = treeGen;
		this.isFlat=isFlat;
	}


	@Override
	public void report(List<EventStatus> statusList)  throws IOException {
		Tree rootTree=Tree.getTree(statusList, treeGen, minimumSeverity, maximumSeverity);
		if(isFlat)
		{
			List<Tree> leafs = new LinkedList<Tree>();
			populateLeafRecursive(rootTree, leafs);
			for (Tree tree : leafs) {
				writeTree(tree, "",true);
			}
		}else
		{
			writeTree(rootTree, "",false);
		}
		writerFooter();
	}

	/**
	 * @param tree
	 * @param tab
	 * @param qualifiedName if true, dump the qualifiedName of the tree. If false, dump its content
	 * @throws IOException
	 */
	protected void writeTree(Tree tree, String tab, boolean qualifiedName) throws IOException
	{
		if(tree.getGroupLevel()>=0)
		{
			writer.append(tab+getContainedStatusRecursive(tree)+ " " + (qualifiedName?tree.getQualifiedName():tree.getContent()));
			tab+="\t";
		}
		for (Tree child : tree.getChildren()) {
			writeTree(child, tab,qualifiedName);
		}
	}
	
	/**
	 * @param tree
	 * @return the number of EventStatus contained by the tree, recursively
	 */
	private int getContainedStatusRecursive(Tree tree)
	{
		int nb=tree.getContainedStatus().size();
		for (Tree child : tree.getChildren()) {
			nb+=getContainedStatusRecursive(child);
		}
		return nb;
	}
	
	/**
	 * Add all leaf tree (branches without children) in leafs list
	 * @param tree
	 * @param leafs
	 */
	private void populateLeafRecursive(Tree tree, List<Tree> leafs)
	{
		if(tree.isLeaf())
			leafs.add(tree);
		else{
			for (Tree child : tree.getChildren()) {
				populateLeafRecursive(child, leafs);
			}
		}
	}
}
