package com.sodius.mdw.log.reporter;

import java.io.IOException;
import java.io.Writer;
import java.util.List;

import org.eclipse.core.runtime.IStatus;

import com.sodius.mdw.log.EventSeverity;
import com.sodius.mdw.log.EventStatus;


/**
 * @author Mickael Albert
 * LogReporter reporting in a Writer
 */
public class TextLogReporter implements LogReporter {

	protected final static String LS=System.getProperty("line.separator");
	protected final EventSeverity minimumSeverity;
	protected final EventSeverity maximumSeverity;
	protected final Writer writer;
	/**
	 *  if true then then the Reporter will also write the severity of each Event
	 */
	protected boolean displaySeverity=true;
	protected final String header;
	protected final String footer;
	
	
	/**
	 * Constructor which set displaySeverity to true
	 * 
	 * @param minimumSeverity Minimum Severity that will be reported
	 * @param maximumSeverity Minimum Severity that will be reported
	 * @param writer Writer on which the reporter should write
	 * @param header Head of the document: directly wrote in the creation of the element
	 * @param footer Tail of the document
	 * @throws IOException
	 */
	public TextLogReporter(EventSeverity minimumSeverity,
			EventSeverity maximumSeverity, Writer writer, String header, String footer) throws IOException {
		this.minimumSeverity = minimumSeverity;
		this.maximumSeverity = maximumSeverity;
		this.writer=writer;
		this.header=header;
		this.footer=footer;
		writerHeader();
	}
	
	protected void writerHeader() throws IOException
	{
		writer.append(header);
	}
	protected void writerFooter() throws IOException
	{
		writer.append(footer);
	}
	
	public TextLogReporter(EventSeverity minimumSeverity,
			EventSeverity maximumSeverity, Writer writer, String header, String footer, boolean displaySeverity) throws IOException {
		this(minimumSeverity, maximumSeverity, writer, header, footer);
		this.displaySeverity=displaySeverity;
	}

	public void report(List<EventStatus> statusList) throws IOException {
		for (EventStatus status : statusList) {
			if(!isFiltered(status))
			{
				dump(status, "");
			}
		}
		writerFooter();
		writer.flush();
	}
	
	/**
	 * @param status
	 * @return true if this status should be not dumped
	 */
	protected boolean isFiltered(EventStatus status)
	{
		return !(status.getEventSeverity().compareTo(minimumSeverity)>=0
				&& status.getEventSeverity().compareTo(maximumSeverity)<=0);
	}
	
	

	/**
	 * @param status
	 * @param tab The text starting each new line
	 * @throws IOException
	 */
	protected void dump(EventStatus status, String tab) throws IOException
	{
		if(displaySeverity)
			writer.append(tab+status.getEventSeverity().name()+' ');
		
		String content = translateLineJumps(status.toString());
		writer.append(content);
		if(!content.endsWith(LS)) //avoid empty lines
			writer.append(LS);
		
		tab+='\t';
		if(status.getException()!=null)
		{
			writer.append(tab+"@Caused by Exception: "+status.getException().getLocalizedMessage());
			writer.append(LS);
		}
		for (IStatus child : status.getChildren()) {
			writer.append(tab+"@Children:");
			writer.append(LS);
			dump((EventStatus)child, tab+'\t');
		}
	}
	
	protected String translateLineJumps(String string)
	{
		return string.replaceAll("\n", LS);
	}
	
	@Override
	protected void finalize() throws Throwable {
		writer.close();
		super.finalize();
	}
	
}
