package com.sodius.rhapsody.toolkit.engine;

import java.io.IOException;
import java.io.InputStream;

import javax.xml.parsers.SAXParser;

import org.eclipse.emf.ecore.xmi.XMIException;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import com.sodius.mdw.core.CoreException;
import com.sodius.mdw.core.util.io.XMLDefaultHandler;
import com.sodius.mdw.core.util.io.XMLUtils;
import com.sodius.mdw.corext.model.io.xmi.XMIHeader;
import com.sodius.mdw.corext.model.io.xmi.XMIVersion;

class RhpXMIHeaderHandler extends XMLDefaultHandler {
    private static enum XMINamespaces {
        XMI_1_0("http://schema.omg.org/spec/XMI/1.0", XMIVersion.XMI_1_0),
        XMI_1_1("http://schema.omg.org/spec/XMI/1.1", XMIVersion.XMI_1_1),
        XMI_1_2("http://schema.omg.org/spec/XMI/1.2", XMIVersion.XMI_1_2),
        XMI_2_0("http://schema.omg.org/spec/XMI/2.0", XMIVersion.XMI_2_0),
        XMI_2_1("http://schema.omg.org/spec/XMI/2.1", XMIVersion.XMI_2_1),
        XMI_2_1_1("http://schema.omg.org/spec/XMI/2.1.1", XMIVersion.XMI_2_1),
        XMI_2_4_1("http://www.omg.org/spec/XMI/20110701", XMIVersion.XMI_2_1),
        XMI_2_4_1_RSA("http://www.omg.org/spec/XMI/2.4.1", XMIVersion.XMI_2_1),
        XMI_2_5("http://www.omg.org/spec/XMI/20131001", XMIVersion.XMI_2_1);
        private final String namespace;
        private final XMIVersion version;

        private XMINamespaces(String namespace, XMIVersion version) {
            this.namespace = namespace;
            this.version = version;
        }

        public String getNamespace() {
            return namespace;
        }

        public XMIVersion getVersion() {
            return version;
        }

        public static XMIVersion getVersion(String namespace) {
            for (XMINamespaces ns : values()) {
                if (ns.getNamespace().equals(namespace)) {
                    return ns.getVersion();
                }
            }
            return null;
        }
    }

    private static class XMIContentReachedException extends Exception {

        /**
         *
         */
        private static final long serialVersionUID = 7506733996089540929L;
    }

    private XMIHeader header;
    private StringBuffer buffer;
    private String nsPrefix;
    private String nsURI;
    private boolean readNextForDocumentation = false;// @See RTC 132921
    private boolean isUML1X = false;
    private boolean isInDocumentation = false;

    public RhpXMIHeaderHandler() {
    }

    protected XMIHeader getHeader() {
        return header;
    }

    protected void setHeader(XMIHeader header) {
        this.header = header;
    }

    protected StringBuffer getBuffer() {
        if (buffer == null) {
            buffer = new StringBuffer();
        }
        return buffer;
    }

    public XMIHeader readHeader(InputStream is) throws IOException, CoreException {
        try {
            setHeader(null);

            SAXParser saxParser = XMLUtils.newParser();
            saxParser.parse(is, this);
            return getHeader();
        } catch (SAXParseException e) {
            throw XMLUtils.createCoreException(e);
        } catch (SAXException e) {
            // header successfully parsed ?
            if (e.getException() instanceof XMIContentReachedException) {
                return getHeader();
            } else {
                throw XMLUtils.createCoreException(e);
            }
        } catch (Exception e) {
            throw new CoreException(e);
        }
    }

    @Override
    public void startElement(String uri, String localName, String qName, Attributes attrs) throws SAXException {
        this.buffer = null;

        // XMI
        if ((getHeader() == null) && (qName.equals(XMI1Constants.XMI_ROOT) || qName.equals(XMI1Constants.XMI_ROOT_FULL))) {
            // 1.x ?
            String versionName = attrs.getValue(XMI1Constants.XMI_VERSION);
            if (versionName == null) {

                // 2.x ?
                versionName = attrs.getValue("xmi:version");
                if (versionName == null) {
                    throw new SAXException(new XMIException("xmi.version attribute not found"));
                }
            } else {
                isUML1X = true;
            }

            // create header
            if (!attemptHeaderCreation(XMIVersion.getVersion(versionName))) {
                throw new SAXException(new CoreException("Unsupported xmi header " + versionName));
            }

            // timestamp
            getHeader().setXMITimestamp(attrs.getValue(XMI1Constants.XMI_TIMESTAMP));
        }

        // XMI.model
        else if (qName.equals(XMI1Constants.XMI_MODEL)) {
            getHeader().setModelName(attrs.getValue("xmi.name"));
            getHeader().setModelVersion(attrs.getValue("xmi.version"));
        }

        // XMI.metamodel
        else if (qName.equals(XMI1Constants.XMI_METAMODEL)) {
            getHeader().setMetamodelName(attrs.getValue("xmi.name"));
            getHeader().setMetamodelVersion(attrs.getValue("xmi.version"));
        }

        // XMI.metametamodel
        else if (qName.equals(XMI1Constants.XMI_METAMETAMODEL)) {
            getHeader().setMetametamodelName(attrs.getValue("xmi.name"));
            getHeader().setMetametamodelVersion(attrs.getValue("xmi.version"));
        }

        else if (isInDocumentation) {
            if (qName.equals("xmi:exporter")) {
                getBuffer().setLength(0);
            }
        } else {
            // xmi 2.x ?
            if (!isUML1X && readNextForDocumentation)// @See RTC 132921
            {
                if (qName.equals("xmi:Documentation")) {
                    isInDocumentation = true;
                    readNextForDocumentation = false;
                    String value = attrs.getValue("exporter"); // For EA
                    if (value != null) {
                        getHeader().setDocExporter(value);
                        throw new SAXException(new XMIContentReachedException()); // nothing else to parse
                    }
                }
            } else {
                if (getHeader() != null) {
                    getHeader().setNamespace(nsPrefix, nsURI);
                    readNextForDocumentation = true;
                }
            }

        }
    }

    private boolean attemptHeaderCreation(XMIVersion version) {
        if (XMIVersion.getSupportedVersions().contains(version)) {
            setHeader(new XMIHeader(version));
            return true;
        }
        return false;
    }

    @Override
    public void startPrefixMapping(String prefix, String uri) throws SAXException {
        if ((getHeader() == null) && prefix.equals("xmi")) {
            XMIVersion versionDetected = XMINamespaces.getVersion(uri);
            if (versionDetected != null) {
                attemptHeaderCreation(versionDetected);
            }
        } else if (prefix.equals("uml")) {
            nsPrefix = prefix;
            nsURI = uri;
        }
    }

    @Override
    public void characters(char[] ch, int start, int length) throws SAXException {
        getBuffer().append(ch, start, length);
    }

    @Override
    public void endElement(String uri, String localName, String qName) throws SAXException {
        // XMI.header
        if (qName.equals(XMI1Constants.XMI_HEADER)) {
            throw new SAXException(new XMIContentReachedException());
        } else if (qName.equals(XMI1Constants.XMI_DOC_OWNER)) {
            getHeader().setDocOwner(getBuffer().toString());
        } else if (qName.equals(XMI1Constants.XMI_DOC_CONTACT)) {
            getHeader().setDocContact(getBuffer().toString());
        } else if (qName.equals(XMI1Constants.XMI_DOC_LONG_DESCRIPTION)) {
            getHeader().setDocLongDescription(getBuffer().toString());
        } else if (qName.equals(XMI1Constants.XMI_DOC_SHORT_DESCRIPTION)) {
            getHeader().setDocShortDescription(getBuffer().toString());
        } else if (qName.equals(XMI1Constants.XMI_DOC_EXPORTER)) {
            getHeader().setDocExporter(getBuffer().toString());
        } else if (qName.equals(XMI1Constants.XMI_DOC_EXPORTER_VERSION)) {
            getHeader().setDocExporterVersion(getBuffer().toString());
        } else if (qName.equals(XMI1Constants.XMI_DOC_EXPORTER_ID)) {
            getHeader().setDocExporterID(getBuffer().toString());
        } else if (qName.equals(XMI1Constants.XMI_DOC_NOTICE)) {
            getHeader().setDocNotice(getBuffer().toString());
        } else if (qName.equals("xmi:exporter")) {
            getHeader().setDocExporter(getBuffer().toString());
        } else if (qName.equals("xmi:Documentation")) {
            isInDocumentation = false;
            throw new SAXException(new XMIContentReachedException()); // nothing else to parse
        }

    }

}