package com.sodius.rhapsody.toolkit.engine;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;

import com.sodius.mdw.core.CoreException;
import com.sodius.mdw.core.OperationCanceledException;
import com.sodius.mdw.core.util.progress.ProgressMonitor;
import com.sodius.mdw.rhapsody.api.utils.RhapsodyAccessor;
import com.sodius.rhapsody.licensing.RhapsodyLicenseManager;
import com.sodius.rhapsody.toolkit.engine.uml21.UML21ExportRunner;
import com.sodius.rhapsody.toolkit.engine.uml21.UML21ImportRunner;
import com.sodius.rhapsody.toolkit.properties.RhapsodyXMIToolkitParameters;
import com.sodius.rhapsody.toolkit.properties.XMIMode;
import com.sodius.rhapsody.toolkit.ui.NoLicenseException;
import com.sodius.rhapsody.toolkit.util.RhapsodyUnitHandler;
import com.sodius.rhapsody.toolkit.util.thread.RhapsodyAPIExec;
import com.telelogic.rhapsody.core.IRPApplication;
import com.telelogic.rhapsody.core.IRPProject;
import com.telelogic.rhapsody.core.RhapsodyAppServer;
import com.telelogic.rhapsody.core.RhapsodyRuntimeException;

public class XMIEngine {

    private IRPApplication rpApplication;
    private final EngineContext context;
    private final RhapsodyXMIToolkitParameters properties;

    private static final int SECRETCODE = 67780580; // this magic number has
                                                    // been given by the
                                                    // Rhapsody development team
    private boolean isPlugin = false;

    public XMIEngine(EngineContext context, int rhapID) {
        this.properties = context.getParameters();
        this.context = context;

        if (rhapID != 0) {
            this.isPlugin = true;
            this.context.getParameters().setProperty("asPlugin", rhapID);
        } else {
            this.context.getParameters().setProperty("asPlugin", 0);
            isPlugin = this.getProperties().getIsPlugin();
        }
    }

    public XMIEngine(EngineContext context, IRPApplication irpApplication) {
        this.properties = context.getParameters();
        this.context = context;
        context.setXMIEngine(this);
        this.rpApplication = irpApplication;
        if (irpApplication != null) {
            this.isPlugin = true;
            this.context.getParameters().setProperty("asPlugin", -1);
        } else {
            this.context.getParameters().setProperty("asPlugin", 0);
            isPlugin = this.getProperties().getIsPlugin();
        }
    }

    public IRPApplication getApplication() {
        // check if rhapsody is still running
        // Just run it here
        if (RhapsodyAPIExec.isInRhapsodyThread()) {
            try {
                basicGetApplication().version();
                return rpApplication;
            } catch (Exception e) {
                try {
                    rpApplication = null;
                    basicGetApplication().version();
                    return rpApplication;
                } catch (Exception e2) {
                    throw new RuntimeException("Unable to connect to a running Rhapsody.\n" + e2.getMessage());
                }
            }
        } else // Run in the Rhapsody Thread
        {
            try {
                RhapsodyAPIExec.synchExec(new Runnable() {

                    public void run() {
                        basicGetApplication().version();
                    }

                });
                return rpApplication;
            } catch (CoreException e) {
                rpApplication = null;
                try {
                    RhapsodyAPIExec.synchExec(new Runnable() {

                        public void run() {
                            basicGetApplication().version();
                        }

                    });
                    return rpApplication;
                } catch (CoreException e2) {
                    throw new RuntimeException("Unable to connect to a running Rhapsody.\n" + e2.getMessage());
                }
            }
        }
    }

    private IRPApplication forceGetNonPluginApplication() {
        IRPApplication forcedApplication = RhapsodyAccessor.getApplication(System.getProperty("os.name").startsWith("Linux"));
        return forcedApplication;
    }

    private IRPApplication basicGetApplication() {
        if (rpApplication == null) {
            int rhpID = this.context.getParameters().getProperty("asPlugin", 0);
            if (rhpID != 0) // is Plugin
            {
                rpApplication = RhapsodyAppServer.attachToRhapsodyApplication(rhpID);
            } else {
                rpApplication = forceGetNonPluginApplication();

            }
        }
        return rpApplication;
    }

    private void clearModel() {
        if ((rpApplication != null) && !isPlugin) {
            try {
                RhapsodyAPIExec.synchExec(new Runnable() {
                    public void run() {
                        rpApplication.unregisterCOMClient(Math.abs(this.hashCode()),
                                new File(getProperties().getToolkitDirectory(), "XMI4Rhapsody.exe").getPath(), SECRETCODE);
                        RhapsodyAppServer.CloseSession();
                    }

                });
            } catch (CoreException e) {
                e.printStackTrace();
            }

        }

        rpApplication = null;

    }

    private void exportProject(ProgressMonitor monitor) {
        try {
            if ((this.getProperties().getSelectedMetamodel() != null) && this.getProperties().getSelectedMetamodel().getUniqueID().equals("uml21")) {

                /*
                 * // separate files ?
                 * boolean separateFiles = getProperties().canUseSeparateFiles() && getProperties().getSeparateFiles();
                 * System.setProperty("separateProfileFiles", separateFiles ? Boolean.TRUE.toString() : "");
                 */

                EngineRunner runner = new UML21ExportRunner(context, getProperties().getToolkitDirectory().getAbsolutePath());
                runner.run(monitor);

            } else {
                context.getOutLogger().error("Selected metamodel doesn't exist");
            }
        } catch (OperationCanceledException e) {
            throw e;
        } catch (Throwable e) {
            context.getOutLogger().error("Error exporting project.", e);
        }
    }

    public IRPProject getActiveProject() throws CoreException {
        final IRPProject[] project = new IRPProject[1];
        RhapsodyAPIExec.synchExec(new Runnable() {

            public void run() {
                project[0] = getApplication().activeProject();
                if (getApplication().activeProject() == null) {
                    throw new RuntimeException("No opened project found in Rhapsody !");
                }
            }

        });

        return project[0];
    }

    private void openProject() {
        try {
            getActiveProject();
        } catch (CoreException e1) {
            if (e1.getCause() instanceof RuntimeException) {
                throw (RuntimeException) e1.getCause();
            } else if (e1.getCause() instanceof RhapsodyRuntimeException) {
                throw new RuntimeException("XMI Import/Export can not be performed in this edition", e1.getCause());
            } else {
                throw new RuntimeException(e1);
            }
        }
    }

    private RhapsodyXMIToolkitParameters getProperties() {
        return properties;
    }

    public void run(final ProgressMonitor monitor) throws Throwable {
        if (this.getProperties().isRhapsodyCL() || this.getProperties().isNoThread()) {
            basicRun(monitor);
        } else {
            RhapsodyAPIExec.synchExec(new Runnable() {

                public void run() {
                    try {
                        basicRun(monitor);
                    } catch (Throwable e) {
                        throw new RuntimeException(e);
                    }
                }
            });
        }
    }

    private void basicRun(ProgressMonitor monitor) throws Throwable {
        String licenseFeature = "RPXMI";
        // checkout license
        if ((getProperties().getMode() == XMIMode.TAU_EXPORT) || (getProperties().getMode() == XMIMode.TAU_IMPORT)) {
            // use tau license
            licenseFeature = "RPTAUXMI";
        }

        // check license
        if (!("".equals(RhapsodyLicenseManager.checkout(licenseFeature, getApplication())))) {
            throw new NoLicenseException(licenseFeature);
        }

        try {
            // check parameters
            if (this.getProperties().isSilentMode()) {
                getProperties().checkSilentParameters();
            }

            if (this.getProperties().isRhapsodyCL() || this.getProperties().isNoThread()) {
                RhapsodyAPIExec.NO_THREAD = true;
            }

            if (monitor.isCanceled()) {
                return;
            }

            // import ?
            if (this.getProperties().isImportMode()) {

                // Manage silent mode unit selection
                String namespacesSelection = properties.getCommandLineProperties().getProperty(properties.KEY_UNIT_SELECTION, "");
                if (namespacesSelection.length() > 0) {
                    properties.setSelectedNamespacesToImport(new ArrayList<Object>(Arrays.asList(namespacesSelection.split("\\,"))));
                    properties.setAllUnitsAreSelected(false);
                }

                // open project
                monitor.beginTask("", 4);
                monitor.worked(1);
                monitor.subTask("Connecting to Rhapsody...");
                this.openProject();
                if (monitor.isCanceled()) {
                    return;
                }

                // import XMI
                monitor.worked(1);
                monitor.subTask("Importing XMI file...");
                if (monitor.isCanceled()) {
                    return;
                }
                // build project
                monitor.worked(1);
                monitor.subTask("Building Rhapsody project...");
                if (monitor.isCanceled()) {
                    return;
                }

                if (this.getProperties().getSelectedMetamodel().getUniqueID().equals("uml21")) {
                    EngineRunner runner = new UML21ImportRunner(context, getProperties().getToolkitDirectory().getAbsolutePath(), getApplication());
                    runner.run(monitor);
                } else {
                    context.getOutLogger().error("Selected metamodel doesn't exist");
                }

                // this.getProperties().setSelectedMetamodel(originalMM);
                monitor.worked(1);
                monitor.subTask("Import Finished.");
            }

            else {
                // Manage silent mode unit selection
                String namespacesSelection = properties.getCommandLineProperties().getProperty(properties.KEY_UNIT_SELECTION, "");
                if (namespacesSelection.length() > 0) {
                    properties.setSelectedUnitsToExport((RhapsodyUnitHandler.createList(namespacesSelection.split("\\,"), getActiveProject())));
                    properties.setAllUnitsAreSelected(false);
                }
                // export
                monitor.beginTask("", 4);
                monitor.worked(1);
                monitor.subTask("Connecting to Rhapsody...");
                if (monitor.isCanceled()) {
                    return;
                }
                this.openProject();

                monitor.worked(1);
                monitor.subTask("Building XMI model...");
                if (monitor.isCanceled()) {
                    return;
                }

                exportProject(monitor);
                monitor.worked(1);
                monitor.subTask("Export Finished.");
            }
        } finally {
            RhapsodyLicenseManager.checkin(licenseFeature, getApplication());
            this.clearModel();
        }
    }

    public EngineContext getContext() {
        return context;
    }

}
