package com.sodius.rhapsody.toolkit.ui;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.lang.reflect.InvocationTargetException;
import java.util.Calendar;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

import com.sodius.mdw.core.CoreException;
import com.sodius.mdw.core.util.log.FileLogger;
import com.sodius.mdw.core.util.log.Logger;
import com.sodius.mdw.core.util.progress.ProgressMonitor;
import com.sodius.mdw.log.EventSeverity;
import com.sodius.mdw.log.EventStackLogger;
import com.sodius.mdw.log.reporter.HTMLTreeLogReporter;
import com.sodius.mdw.log.reporter.SummarizedLogReporter;
import com.sodius.mdw.log.reporter.TextLogReporter;
import com.sodius.mdw.log.reporter.tree.ModelSubTreeGenerator;
import com.sodius.rhapsody.toolkit.XMIToolkitApplication;
import com.sodius.rhapsody.toolkit.engine.EngineContext;
import com.sodius.rhapsody.toolkit.engine.XMIEngine;
import com.sodius.rhapsody.toolkit.properties.RhapsodyXMIToolkitParameters;
import com.sodius.rhapsody.toolkit.util.NullLogger;
import com.sodius.rhapsody.toolkit.util.PrintStreamLogger;
import com.sodius.rhapsody.toolkit.util.ReportTreeGenerator;
import com.sodius.rhapsody.toolkit.util.thread.RhapsodyAPIExec;
import com.telelogic.rhapsody.core.IRPApplication;

public class RhapsodyXMIToolkit implements EngineContext {

	private XMIEngine engine=null;
	private PrintStream defaultSysOut=null;
	private PrintStream defaultSysErr=null;
	private RhapsodyConsoleWriter rhapsodyConsoleWriter;

	// properties
	private RhapsodyXMIToolkitParameters properties;

	// console
	private Logger outLogger;
	private Logger errLogger;
	private RhapsodyConsolLogger outRhapsodyLogger;

	protected static final String VALIDATION_PROPERTY = "http://xml.org/sax/features/validation";

	public String[] languages = { "", "Java", "Ada", "C++", "C" };
	private ToolkitUIManager manager;

	public RhapsodyXMIToolkit(String[] args) {
		this(args, 0);
	}
	public RhapsodyXMIToolkit(String[] args, IRPApplication irpApplication) {
		this(args, new RhapsodyXMIToolkitParameters(), irpApplication);
	}

	public RhapsodyXMIToolkit(String[] args, int rhpID) {
		this(args, new RhapsodyXMIToolkitParameters(), rhpID);
	}

	private RhapsodyXMIToolkit(String[] args, RhapsodyXMIToolkitParameters properties, int rhpID) {
		try {
			this.properties = properties;
			getProperties().read(args);
			engine = new XMIEngine(this, rhpID);
			initialize();
		} catch (Throwable e) {
			MessageDialog.openError(new Shell(), getToolkitName(), "Unable to start " + getToolkitName() + " :\n\n" + e.toString());
		}
	}
	private RhapsodyXMIToolkit(String[] args, RhapsodyXMIToolkitParameters properties, IRPApplication irpApplication) {
		try {
			this.properties = properties;
			getProperties().read(args);
			engine = new XMIEngine(this, irpApplication);
			initialize();
		} catch (Throwable e) {
			MessageDialog.openError(new Shell(), getToolkitName(), "Unable to start " + getToolkitName() + " :\n\n" + e.toString());
		}
	}

	private void createBasicLoggers()
	{
		outRhapsodyLogger=new RhapsodyConsolLogger(engine);
		File logout = new File(getProperties().getLogDirectory(), "system_out.txt");
		File logerr = new File(getProperties().getLogDirectory(), "system_err.txt");
		try{
			outLogger = new FileLogger(logout);
			errLogger = new FileLogger(logerr);
		}catch (Exception e) {
			outLogger=new NullLogger();
			errLogger=new NullLogger();
			e.printStackTrace();
		}
	}
	
	public Logger getDirectOutLogger() {
		if(outRhapsodyLogger!=null)
			return outRhapsodyLogger;
		else
			return outLogger;
	}
	
	private void createEventStackLogger(Shell shell)
	{
		rhapsodyConsoleWriter=new RhapsodyConsoleWriter(engine);
		File logout = new File(getProperties().getLogDirectory(), "system_out.txt");
		File logerr = new File(getProperties().getLogDirectory(), "system_err.txt");
		File logHtml=new File(getProperties().getLogDirectory(), "log.html");
		EventStackLogger logLogger = new EventStackLogger();
		//add sysout
		try {
			logLogger.registerReporter(new TextLogReporter(EventSeverity.INFO, EventSeverity.WARNING, new PrintWriter(logout),"","",false));
		} catch (Exception e) {
			showErrorDialog(shell, "Impossible to create log " + logout.getAbsolutePath() + ". Due to error " + e.getMessage());
		}
		//add syserr
		try {
			logLogger.registerReporter(new TextLogReporter(EventSeverity.ERROR, EventSeverity.FATAL, new PrintWriter(logerr),"","",false));
		} catch (Exception e) {
			showErrorDialog(shell, "Impossible to create log " + logerr.getAbsolutePath() + ". Due to error " + e.getMessage());
		}
		//add html
		
		try {
			logLogger.registerReporter(new HTMLTreeLogReporter(EventSeverity.INFO, EventSeverity.FATAL, 
					new FileWriter(logHtml),
					"<T1>IBM Rhapsody XMIToolkit logs " + Calendar.getInstance().getTime().toString() + "</T1><BR>",
					"End of log<BR>",
					"XMIToolkit report", 
					new File(getProperties().getResourcesDirectory(), "htmlReport"), new ReportTreeGenerator(false,false), new ModelSubTreeGenerator(false)));
		} catch (Exception e) {
			showErrorDialog(shell, "Impossible to create log " + logHtml.getAbsolutePath() + ". Due to error " + e.getMessage());
		}
		//add Rhapsody Console summary
		try {
			logLogger.registerReporter(new SummarizedLogReporter(EventSeverity.WARNING, EventSeverity.FATAL, rhapsodyConsoleWriter,
					"Launching XMIToolkit...",
					"Execution done.",
					new ReportTreeGenerator(false,true), false));
		} catch (Exception e) {
			showErrorDialog(shell, "Impossible to create log in Rhapsody Console. Due to error " + e.getMessage());
		}
		errLogger=logLogger;
		outLogger=logLogger;
	}

	private RhapsodyXMIToolkitParameters getProperties() {
		return properties;
	}

	public File getToolkitDirectory() {
		return getProperties().getToolkitDirectory();
	}

	private void initialize() {
		// loggers
		if(getParameters().getAdvancedLogger())
			createEventStackLogger(null);
		else
			createBasicLoggers();
		setSystemStreams();
	}

	void cleanAtEnd(){

		try {
			// loggers
			unsetSystemStreams();
			RhapsodyAPIExec.synchExec(RhapsodyAPIExec.FINISH_THREAD);
		} catch (CoreException e) {
			e.printStackTrace();
		}
	}


	private void setSystemStreams() {
		try {
			// out
			this.defaultSysOut=System.out;
			System.setOut(new PrintStreamLogger(getOutLogger(), false));

			// err
			this.defaultSysErr=System.err;
			System.setErr(new PrintStreamLogger(getErrLogger(), true));
		} catch (Throwable e) {
			showErrorDialog(null, "Cannot attach logger to System.out / Sytem.err due to error " + e.getMessage());
		}
	}
	private void unsetSystemStreams() {
		outLogger=null;
		errLogger=null;
		try {
			// out
			if(this.defaultSysOut!=null)
				System.setOut(this.defaultSysOut);
			
			// err
			if(this.defaultSysErr!=null)
				System.setErr(this.defaultSysErr);
		} catch (Throwable e) {
			e.printStackTrace();
		}
	}

	private void flushEventStackLogger(boolean onlyWithStack) throws IOException
	{
		if(outLogger instanceof EventStackLogger)
			((EventStackLogger)outLogger).flush(onlyWithStack);
	}
	
	private void showInformationMessage(Shell shell, String message) {
		MessageDialog.openInformation(shell, getToolkitName(), message);
	}
	private void showErrorMessage(Shell shell, String message) {
		MessageDialog.openError(shell, getToolkitName(), message);
	}

	public void actionProceed(ProgressMonitorDialog monitorDialog) {
		if(!getParameters().isCommandLineProperty(RhapsodyXMIToolkitParameters.KEY_XMI_FILENAME))
			getParameters().setXMIFilename(manager.getXMIFilename());
		basicActionProceed(monitorDialog);
	}

	public RhapsodyXMIToolkitParameters getParameters() {
		return (RhapsodyXMIToolkitParameters) getProperties();
	}

	private void basicActionProceed(ProgressMonitorDialog monitorDialog) {
		// run

		try {
			monitorDialog.run(true, true, new IRunnableWithProgress() {

				public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
					try {
						XMITask task = new XMITask(RhapsodyXMIToolkit.this, engine);
						EclipseProgressMonitor eclipseProgressMonitor = new EclipseProgressMonitor(monitor);
						task.run((ProgressMonitor) eclipseProgressMonitor);
					}
					catch (Throwable e) {
						throw new InvocationTargetException(e);
					}
				}
			});
			
			try {
				this.getParameters().save();
			} catch (IOException e) {
				e.printStackTrace();
			}
			flushEventStackLogger(false);
			if (isEmptyOutLog()){
				showCompletedDialog(null);
			}else{
				showErrorsDialog(null);
			}
		} catch (InvocationTargetException e) {
			XMIToolkitApplication.displayErrorUI(Display.getDefault().getActiveShell(),e);
		} catch (InterruptedException e) {
			MessageDialog.openInformation(new Shell(), "Cancelled", e.getMessage());
		} catch (IOException e) {
			showErrorDialog(null, "Impossible to generate logs due to error "+e.getMessage());
		} finally{
			try {
				flushEventStackLogger(true);
			} catch (IOException e) {
				showErrorDialog(null, "Impossible to generate logs due to error "+e.getMessage());
			}
		}
	}

	
	
	public RhapsodyXMIToolkitParameters getRhapsodyParameters() {
		return (RhapsodyXMIToolkitParameters) getProperties();
	}

	public String getToolkitName() {
		return "XMI Toolkit for Rhapsody"; //TODO : change name following import or export
	}

	public void setManager(ToolkitUIManager manager) {
		this.manager = manager;

	}

	public Logger getErrLogger() {
		return errLogger;
	}

	public Logger getOutLogger() {
		return outLogger;
	}

	public XMIEngine getEngine() {
		return engine;
	}

	public boolean isEmptyOutLog() {
		if(getParameters().getAdvancedLogger())
			return rhapsodyConsoleWriter.isEmpty();
		else
			return outRhapsodyLogger.isEmpty();
	}
	
	private void showErrorsDialog(Shell shell) {
		showErrorDialog(shell, (getParameters().isImportMode()?"Import":"Export") + " completed, but there are warnings !");
	}

	private void showCompletedDialog(Shell shell) {
		// silent "Export or Import complete"
		String showDialog = getParameters().getCommandLineProperties().getProperty(RhapsodyXMIToolkitParameters.KEY_SHOW_INFO_DIALOG, "true");
		if("true".equalsIgnoreCase(showDialog)){
			showInformationMessage(shell, (getParameters().isImportMode()?"Import":"Export") + " completed");
		}
	}
	
	private void showErrorDialog(Shell shell, String message)
	{
		String showDialog = getParameters().getCommandLineProperties().getProperty(RhapsodyXMIToolkitParameters.KEY_SHOW_INFO_DIALOG, "true");
		if("true".equalsIgnoreCase(showDialog)){
			showErrorMessage(shell, message);
		}
	}

	public XMIEngine getXMIEngine() {
		return engine;
	}
	public void setXMIEngine(XMIEngine engine) {
		this.engine=engine;
	}

}