package com.sodius.rhapsody.toolkit.ui;

import java.io.File;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import com.sodius.mdw.core.CoreException;
import com.sodius.rhapsody.toolkit.engine.MetamodelChecker;
import com.sodius.rhapsody.toolkit.properties.RhapsodyXMIToolkitParameters;
import com.sodius.rhapsody.toolkit.properties.UMLVersionKind;

public class ShowFileControl extends Composite {

    private final ToolkitUIManager manager;
    private final WizardPage page;
    private final Label title;
    private final Composite saveFileComposite;
    private Text fileName;
    private SelectionListener fileNameSelectionListener;
    private UMLVersionKind importMetamodel = null;
    private static final Set<String> UMLExtensionsWritable = new HashSet<String>();
    static {
        UMLExtensionsWritable.add("xmi");
        UMLExtensionsWritable.add("uml");
        UMLExtensionsWritable.add("uml2");
    }

    ShowFileControl(Composite parent, WizardPage page, ToolkitUIManager manager, String label) {
        super(parent, SWT.NONE);
        this.page = page;
        this.setLayout(new GridLayout(1, false));
        this.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        this.manager = manager;
        title = new Label(this, SWT.NONE);
        title.setText(label);

        saveFileComposite = new Composite(this, SWT.NONE);
        saveFileComposite.setLayout(new GridLayout(2, false));
        saveFileComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        createContents();
    }

    public void createContents() {
        fileName = new Text(saveFileComposite, SWT.BORDER);
        GridData data = new GridData(GridData.FILL_HORIZONTAL);
        data.widthHint = 500;
        fileName.setLayoutData(data);
        fileName.addHelpListener(manager.getHelpListener());

        Button save = new Button(saveFileComposite, SWT.PUSH);
        save.setText("Browse...");
        fileNameSelectionListener = new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent event) {
                int style;
                if (manager.isImportMode()) {
                    style = SWT.OPEN;
                } else {
                    style = SWT.SAVE;
                }
                FileDialog dlg = new FileDialog(ShowFileControl.this.getShell(), style);
                String[] FILTER_NAMES;
                String[] FILTER_EXTS;
                if (manager.isImportMode()) {
                    FILTER_NAMES = new String[] { "XMI files (*.xmi, *.xml)", "RSx emx files (*.emx)", "RSx epx files (*.epx)",
                            "Eclipse UML files (*.uml)", "All files (*.*)" };
                    FILTER_EXTS = new String[] { "*.xmi; *.xml", "*.emx", "*.epx", "*.uml", "*.*" };
                } else {
                    FILTER_NAMES = new String[] { "XMI files (*.xmi, *.xml)", "Eclipse UML files (*.uml)", "All files (*.*)" };
                    FILTER_EXTS = new String[] { "*.xmi; *.xml", "*.uml", "*.*" };
                }
                dlg.setFilterNames(FILTER_NAMES);
                dlg.setFilterExtensions(FILTER_EXTS);
                String fn = dlg.open();
                if (fn != null) {
                    fileName.setText(fn);
                }
            }
        };
        save.addSelectionListener(fileNameSelectionListener);
        fileName.addModifyListener(new ModifyListener() {

            public void modifyText(ModifyEvent e) {
                checkFileValidy();
            }
        });
        save.addHelpListener(manager.getHelpListener());

        String lastFileName = manager.getToolkit().getParameters().getXMIFilename();
        fileName.setText(lastFileName != null ? lastFileName : ""); // @See RTC 124781
        String message = computeURIErrorMessage(fileName.getText().trim());
        this.layout(true, true);
        this.pack();
        if (message != null) {
            manager.getDialog().setErrorMessage(message);
        }
    }

    void checkFileValidy() {
        String filePath = fileName.getText().trim();
        String message = computeURIErrorMessage(filePath);
        if ((message == null) && (filePath.length() > 0)) {
            manager.setXmiFilename(filePath);
            if (!manager.isImportMode()) {
                page.setPageComplete(true);
            }
        } else {
            if (!manager.isImportMode()) {
                page.setPageComplete(false);
            }
        }
        manager.getDialog().setErrorMessage(message);
    }

    private static final String errorDirectory = "This file is a directory";
    private static final String errorNotExist = "This file does not exist";
    private static final String errorNotAbsolute = "The file path must be absolute";
    private static final String errorMetamodel = "Cannot determine UML version";
    private static final String errorNotUMLExtensionWritable = "The file does not contain valid UML extension (.xmi, .uml, .uml2)";

    public String computeURIErrorMessage(String resolvedURI) {
        File file = new File(resolvedURI);

        // open
        if (manager.isImportMode()) {

            // ! exists ?
            if (!file.exists()) {
                importMetamodel = null;
                return errorNotExist;
            }

            // ! absolute ?
            if (!file.isAbsolute()) {
                importMetamodel = null;
                return errorNotAbsolute;
            }

            try {
                String[] metamodel = new MetamodelChecker(manager.getToolkit()).getMetamodelNamedInXMI(resolvedURI);
                setMetamodel(metamodel);
                if (importMetamodel == null) {
                    return errorMetamodel;
                }
            } catch (CoreException e) {
                importMetamodel = null;
                return e.getMessage();
            }
            // save
        } else {
            // directory ?
            if (file.isDirectory()) {
                return errorDirectory;
            } else if (!file.isAbsolute()) {
                return errorNotAbsolute;
            } else if (!isUMLExtensionWritable(file)) {
                return errorNotUMLExtensionWritable;
            }

        }

        return null;
    }

    private boolean isUMLExtensionWritable(File file) {
        int index = file.getName().lastIndexOf('.');
        if (index >= 0) {
            return UMLExtensionsWritable.contains(file.getName().substring(index + 1).toLowerCase());
        }
        return false;
    }

    private void setMetamodel(String[] metamodel) {
        if ((metamodel == null) || (metamodel.length < 2) || (metamodel[0] != "UML")) {
            importMetamodel = null;
            return;
        }
        RhapsodyXMIToolkitParameters parameters = manager.getToolkit().getParameters();
        if (metamodel[1] == "2.1") {
            if ((metamodel.length > 2) && (metamodel[2] == "EA") && manager.getToolkit().getParameters().getEnableUMLEA()) {
                importMetamodel = UMLVersionKind.uml21EA;
            } else if ((metamodel.length > 2) && (metamodel[2] == "MD") && manager.getToolkit().getParameters().getEnableUMLMD()) {
                importMetamodel = UMLVersionKind.uml21MD;
            } else {
                importMetamodel = UMLVersionKind.uml21;
            }
            parameters.setSelectedMetamodel(parameters.getMetamodelFromCommandLineFormat(importMetamodel), importMetamodel);
        }

    }

    public Text getFileName() {
        return fileName;
    }

    public SelectionListener getFileNameSelectionListener() {
        return fileNameSelectionListener;
    }

    public Label getTitle() {
        return title;
    }

    public UMLVersionKind getImportMetamodel() {
        return importMetamodel;
    }

}